@@ -56,26 +56,41 @@ __all__ = (
 ArrayValueTypeDef = TypedDict(
     "ArrayValueTypeDef",
     {
-        "arrayValues": NotRequired[Sequence[Dict[str, Any]]],
-        "booleanValues": NotRequired[Sequence[bool]],
-        "doubleValues": NotRequired[Sequence[float]],
-        "longValues": NotRequired[Sequence[int]],
-        "stringValues": NotRequired[Sequence[str]],
+        "arrayValues": Sequence[Dict[str, Any]],
+        "booleanValues": Sequence[bool],
+        "doubleValues": Sequence[float],
+        "longValues": Sequence[int],
+        "stringValues": Sequence[str],
     },
+    total=False,
 )
 
-BatchExecuteStatementRequestRequestTypeDef = TypedDict(
-    "BatchExecuteStatementRequestRequestTypeDef",
+_RequiredBatchExecuteStatementRequestRequestTypeDef = TypedDict(
+    "_RequiredBatchExecuteStatementRequestRequestTypeDef",
     {
         "resourceArn": str,
         "secretArn": str,
         "sql": str,
-        "database": NotRequired[str],
-        "parameterSets": NotRequired[Sequence[Sequence["SqlParameterTypeDef"]]],
-        "schema": NotRequired[str],
-        "transactionId": NotRequired[str],
     },
 )
+_OptionalBatchExecuteStatementRequestRequestTypeDef = TypedDict(
+    "_OptionalBatchExecuteStatementRequestRequestTypeDef",
+    {
+        "database": str,
+        "parameterSets": Sequence[Sequence["SqlParameterTypeDef"]],
+        "schema": str,
+        "transactionId": str,
+    },
+    total=False,
+)
+
+
+class BatchExecuteStatementRequestRequestTypeDef(
+    _RequiredBatchExecuteStatementRequestRequestTypeDef,
+    _OptionalBatchExecuteStatementRequestRequestTypeDef,
+):
+    pass
+
 
 BatchExecuteStatementResponseTypeDef = TypedDict(
     "BatchExecuteStatementResponseTypeDef",
@@ -85,15 +100,28 @@ BatchExecuteStatementResponseTypeDef = TypedDict(
     },
 )
 
-BeginTransactionRequestRequestTypeDef = TypedDict(
-    "BeginTransactionRequestRequestTypeDef",
+_RequiredBeginTransactionRequestRequestTypeDef = TypedDict(
+    "_RequiredBeginTransactionRequestRequestTypeDef",
     {
         "resourceArn": str,
         "secretArn": str,
-        "database": NotRequired[str],
-        "schema": NotRequired[str],
     },
 )
+_OptionalBeginTransactionRequestRequestTypeDef = TypedDict(
+    "_OptionalBeginTransactionRequestRequestTypeDef",
+    {
+        "database": str,
+        "schema": str,
+    },
+    total=False,
+)
+
+
+class BeginTransactionRequestRequestTypeDef(
+    _RequiredBeginTransactionRequestRequestTypeDef, _OptionalBeginTransactionRequestRequestTypeDef
+):
+    pass
+
 
 BeginTransactionResponseTypeDef = TypedDict(
     "BeginTransactionResponseTypeDef",
@@ -106,21 +134,22 @@ BeginTransactionResponseTypeDef = TypedDict(
 ColumnMetadataTypeDef = TypedDict(
     "ColumnMetadataTypeDef",
     {
-        "arrayBaseColumnType": NotRequired[int],
-        "isAutoIncrement": NotRequired[bool],
-        "isCaseSensitive": NotRequired[bool],
-        "isCurrency": NotRequired[bool],
-        "isSigned": NotRequired[bool],
-        "label": NotRequired[str],
-        "name": NotRequired[str],
-        "nullable": NotRequired[int],
-        "precision": NotRequired[int],
-        "scale": NotRequired[int],
-        "schemaName": NotRequired[str],
-        "tableName": NotRequired[str],
-        "type": NotRequired[int],
-        "typeName": NotRequired[str],
+        "arrayBaseColumnType": int,
+        "isAutoIncrement": bool,
+        "isCaseSensitive": bool,
+        "isCurrency": bool,
+        "isSigned": bool,
+        "label": str,
+        "name": str,
+        "nullable": int,
+        "precision": int,
+        "scale": int,
+        "schemaName": str,
+        "tableName": str,
+        "type": int,
+        "typeName": str,
     },
+    total=False,
 )
 
 CommitTransactionRequestRequestTypeDef = TypedDict(
@@ -140,16 +169,29 @@ CommitTransactionResponseTypeDef = TypedDict(
     },
 )
 
-ExecuteSqlRequestRequestTypeDef = TypedDict(
-    "ExecuteSqlRequestRequestTypeDef",
+_RequiredExecuteSqlRequestRequestTypeDef = TypedDict(
+    "_RequiredExecuteSqlRequestRequestTypeDef",
     {
         "awsSecretStoreArn": str,
         "dbClusterOrInstanceArn": str,
         "sqlStatements": str,
-        "database": NotRequired[str],
-        "schema": NotRequired[str],
     },
 )
+_OptionalExecuteSqlRequestRequestTypeDef = TypedDict(
+    "_OptionalExecuteSqlRequestRequestTypeDef",
+    {
+        "database": str,
+        "schema": str,
+    },
+    total=False,
+)
+
+
+class ExecuteSqlRequestRequestTypeDef(
+    _RequiredExecuteSqlRequestRequestTypeDef, _OptionalExecuteSqlRequestRequestTypeDef
+):
+    pass
+
 
 ExecuteSqlResponseTypeDef = TypedDict(
     "ExecuteSqlResponseTypeDef",
@@ -159,21 +201,34 @@ ExecuteSqlResponseTypeDef = TypedDict(
     },
 )
 
-ExecuteStatementRequestRequestTypeDef = TypedDict(
-    "ExecuteStatementRequestRequestTypeDef",
+_RequiredExecuteStatementRequestRequestTypeDef = TypedDict(
+    "_RequiredExecuteStatementRequestRequestTypeDef",
     {
         "resourceArn": str,
         "secretArn": str,
         "sql": str,
-        "continueAfterTimeout": NotRequired[bool],
-        "database": NotRequired[str],
-        "includeResultMetadata": NotRequired[bool],
-        "parameters": NotRequired[Sequence["SqlParameterTypeDef"]],
-        "resultSetOptions": NotRequired["ResultSetOptionsTypeDef"],
-        "schema": NotRequired[str],
-        "transactionId": NotRequired[str],
     },
 )
+_OptionalExecuteStatementRequestRequestTypeDef = TypedDict(
+    "_OptionalExecuteStatementRequestRequestTypeDef",
+    {
+        "continueAfterTimeout": bool,
+        "database": str,
+        "includeResultMetadata": bool,
+        "parameters": Sequence["SqlParameterTypeDef"],
+        "resultSetOptions": "ResultSetOptionsTypeDef",
+        "schema": str,
+        "transactionId": str,
+    },
+    total=False,
+)
+
+
+class ExecuteStatementRequestRequestTypeDef(
+    _RequiredExecuteStatementRequestRequestTypeDef, _OptionalExecuteStatementRequestRequestTypeDef
+):
+    pass
+
 
 ExecuteStatementResponseTypeDef = TypedDict(
     "ExecuteStatementResponseTypeDef",
@@ -189,21 +244,23 @@ ExecuteStatementResponseTypeDef = TypedDict(
 FieldTypeDef = TypedDict(
     "FieldTypeDef",
     {
-        "arrayValue": NotRequired["ArrayValueTypeDef"],
-        "blobValue": NotRequired[Union[bytes, IO[bytes], StreamingBody]],
-        "booleanValue": NotRequired[bool],
-        "doubleValue": NotRequired[float],
-        "isNull": NotRequired[bool],
-        "longValue": NotRequired[int],
-        "stringValue": NotRequired[str],
+        "arrayValue": "ArrayValueTypeDef",
+        "blobValue": Union[bytes, IO[bytes], StreamingBody],
+        "booleanValue": bool,
+        "doubleValue": float,
+        "isNull": bool,
+        "longValue": int,
+        "stringValue": str,
     },
+    total=False,
 )
 
 RecordTypeDef = TypedDict(
     "RecordTypeDef",
     {
-        "values": NotRequired[List["ValueTypeDef"]],
+        "values": List["ValueTypeDef"],
     },
+    total=False,
 )
 
 ResponseMetadataTypeDef = TypedDict(
@@ -220,24 +277,27 @@ ResponseMetadataTypeDef = TypedDict(
 ResultFrameTypeDef = TypedDict(
     "ResultFrameTypeDef",
     {
-        "records": NotRequired[List["RecordTypeDef"]],
-        "resultSetMetadata": NotRequired["ResultSetMetadataTypeDef"],
+        "records": List["RecordTypeDef"],
+        "resultSetMetadata": "ResultSetMetadataTypeDef",
     },
+    total=False,
 )
 
 ResultSetMetadataTypeDef = TypedDict(
     "ResultSetMetadataTypeDef",
     {
-        "columnCount": NotRequired[int],
-        "columnMetadata": NotRequired[List["ColumnMetadataTypeDef"]],
+        "columnCount": int,
+        "columnMetadata": List["ColumnMetadataTypeDef"],
     },
+    total=False,
 )
 
 ResultSetOptionsTypeDef = TypedDict(
     "ResultSetOptionsTypeDef",
     {
-        "decimalReturnType": NotRequired[DecimalReturnTypeType],
+        "decimalReturnType": DecimalReturnTypeType,
     },
+    total=False,
 )
 
 RollbackTransactionRequestRequestTypeDef = TypedDict(
@@ -260,46 +320,51 @@ RollbackTransactionResponseTypeDef = TypedDict(
 SqlParameterTypeDef = TypedDict(
     "SqlParameterTypeDef",
     {
-        "name": NotRequired[str],
-        "typeHint": NotRequired[TypeHintType],
-        "value": NotRequired["FieldTypeDef"],
+        "name": str,
+        "typeHint": TypeHintType,
+        "value": "FieldTypeDef",
     },
+    total=False,
 )
 
 SqlStatementResultTypeDef = TypedDict(
     "SqlStatementResultTypeDef",
     {
-        "numberOfRecordsUpdated": NotRequired[int],
-        "resultFrame": NotRequired["ResultFrameTypeDef"],
+        "numberOfRecordsUpdated": int,
+        "resultFrame": "ResultFrameTypeDef",
     },
+    total=False,
 )
 
 StructValueTypeDef = TypedDict(
     "StructValueTypeDef",
     {
-        "attributes": NotRequired[List[Dict[str, Any]]],
+        "attributes": List[Dict[str, Any]],
     },
+    total=False,
 )
 
 UpdateResultTypeDef = TypedDict(
     "UpdateResultTypeDef",
     {
-        "generatedFields": NotRequired[List["FieldTypeDef"]],
+        "generatedFields": List["FieldTypeDef"],
     },
+    total=False,
 )
 
 ValueTypeDef = TypedDict(
     "ValueTypeDef",
     {
-        "arrayValues": NotRequired[List[Dict[str, Any]]],
-        "bigIntValue": NotRequired[int],
-        "bitValue": NotRequired[bool],
-        "blobValue": NotRequired[bytes],
-        "doubleValue": NotRequired[float],
-        "intValue": NotRequired[int],
-        "isNull": NotRequired[bool],
-        "realValue": NotRequired[float],
-        "stringValue": NotRequired[str],
-        "structValue": NotRequired[Dict[str, Any]],
+        "arrayValues": List[Dict[str, Any]],
+        "bigIntValue": int,
+        "bitValue": bool,
+        "blobValue": bytes,
+        "doubleValue": float,
+        "intValue": int,
+        "isNull": bool,
+        "realValue": float,
+        "stringValue": str,
+        "structValue": Dict[str, Any],
     },
+    total=False,
 )
